package org.tip.puck.spacetime;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.tip.puck.net.relations.Relation;
import org.tip.puck.spacetime.workers.SequenceCensus;
import org.tip.puck.spacetime.workers.SpaceTimeCriteria.RelationClassificationType;

/**
 * 
 * @author Klaus Hamberger
 *
 */
public class SequenceProfile {
	
	int nrLoops;
	int nrDirectReturns;
	int nrCycles;
	List<String> profile;
	Set<String> support;
	Set<String> centers;
	String start;
	
	
	public SequenceProfile(List<Relation> events, Map<RelationClassificationType,Map<Relation,String>> map, RelationClassificationType type){
		
		profile = new ArrayList<String>();
		support = new HashSet<String>();
		centers = new HashSet<String>();
			
		int i=0;
		for (Relation event : events){
			if (SequenceCensus.isBirth(event)){
				start = map.get(type).get(event);
			}
			
			if (!SequenceCensus.isBirth(event) || type == RelationClassificationType.PLACE){
				
				String place = map.get(type).get(event);
				profile.add(place);
				support.add(place);
				
				// Check return pattern
				String previousPlace = null;
				boolean closed = false;
				
				for (int  j=i-1;j>-1;j--){
					previousPlace = profile.get(j);
					closed = previousPlace!=null && previousPlace.equals(place);
					if (closed){
						centers.add(place);
						if (j == (i-1)){
							nrLoops++;
						} else if (j == (i-2)){
							nrDirectReturns++;
						} else {
							nrCycles++;
						}
						break;
					}
				}
			}
			i++;
		}
	}
	
	public List<String> getCentersAsList(){
		List<String> result;
		
		result = new ArrayList<String>(centers);
		Collections.sort(result);
		
		//
		return result;
	}

	public List<String> getCentersWithoutStartAsList(){
		List<String> result;
		
		result = new ArrayList<String>(centers);
		result.remove(start);
		Collections.sort(result);
		
		//
		return result;
	}

	public Set<String> getSupport() {
		return support;
	}

	public List<String> getSupportAsList(){
		List<String> result;
		
		result = new ArrayList<String>(support);
		Collections.sort(result);
		
		//
		return result;
	}

	public int getNrDirectReturns() {
		return nrDirectReturns;
	}

	public int getNrCycles() {
		return nrCycles;
	}

	public int getNrLoops() {
		return nrLoops;
	}

	public List<String> getProfile() {
		return profile;
	}
	
	public int getNrCenters(){
		return centers.size();
	}
	
	public int getNrStations(){
		return support.size();
	}
	
	public int getNrCentersWithoutStart(){
		int result;
		
		result = getNrCenters();
		if (centers.contains(start)){
			result = result-1;
		}
		//
		return result;
	}
	
	

}
